//******************************************************************************
//
//  g4fon Reflectometer or SWR bridge...
//  
//******************************************************************************

#include "16C71.h"
#include "display.h"
#define p16c71

#fuses HS,NOWDT,NOPROTECT

#use delay(clock=4000000)
#use fast_io (B)

//******************************************************************************
//
//  LCD Support routines...
//  
//******************************************************************************

void
LCD_Wait_Busy ()
	{
	char temp;
	int x;

    //  set the upper nibble to input...
    
   	set_tris_b ( 0xf0 );

    //  set up the control signals...
    
    output_low  ( LCD_RS );
   	output_high ( LCD_RW );

    //  read in the status, nibble at a time
    //  and test that bit 7 is zero...
    //  But don't loop for ever doing it...
    
    x = 0;

	do
	    {
        delay_us ( 5 );
        
	    output_high ( LCD_EN );

        temp = portb & 0xf0;

	    output_low ( LCD_EN );

        delay_us ( 5 );
        
	    output_high ( LCD_EN );

        temp |= swap ( portb & 0xf0 );

	    output_low ( LCD_EN );
	    }
    while ( bit_test ( temp, 7 ) && ( ++x < 100 ));

    //  reset the control signal
    
    output_low ( LCD_RW );

    //  and the output direction
    
   	set_tris_b ( 0 );
    }
	
//******************************************************************************
//
//  The character is sent nibble at a time, top nibble first...
//

void
LCD_Send_LCD ( char c )
	{
    Portb = (portb & 0x0F ) + ( c & 0xf0 );
    
	output_bit ( LCD_EN, 1 );

    delay_us ( 5 );
        
	output_bit ( LCD_EN, 0 );

    swap ( c );
    
    Portb = (portb & 0x0f ) + ( c & 0xf0 );
    
	output_bit ( LCD_EN, 1 );

    delay_us ( 5 );
        
	output_bit ( LCD_EN, 0 );
	}
	
//******************************************************************************

void
LCD_Send_Char ( char c )
	{
    //  wait until the display is listening...
    
	LCD_Wait_Busy ();

	//  set up the control signals.
	
	output_bit ( LCD_RS, 1 );
	output_bit ( LCD_RW, 0 );


    LCD_Send_LCD ( c );
	}
	
//******************************************************************************

void
LCD_Send_Cmd ( char c )
	{
    //  wait for the display to listen to us
    
	LCD_Wait_Busy ();

	//  set up the control signals.
	
	output_bit ( LCD_RS, 0 );
	output_bit ( LCD_RW, 0 );

    LCD_Send_LCD ( c );
	}
	
//******************************************************************************

void
LCD_Init ()
	{
	output_bit ( LCD_EN, 0 );
	output_bit ( LCD_RS, 0 );
	output_bit ( LCD_RW, 0 );

	delay_ms ( 20 );
	set_tris_b ( 0 );
	
	porta = 0x30;

	output_bit ( LCD_EN, 1 );
	output_bit ( LCD_EN, 0 );

	delay_ms ( 5 );
	
	output_bit ( LCD_EN, 1 );
	output_bit ( LCD_EN, 0 );

    delay_us ( 150 );
    
	output_bit ( LCD_EN, 1 );
	output_bit ( LCD_EN, 0 );

    LCD_Wait_Busy ();

    LCD_Send_Cmd ( 0x20 );
    LCD_Send_Cmd ( 0x28 );
    LCD_Send_Cmd ( 0x08 );
    LCD_Send_Cmd ( 0x0c );
    LCD_Send_Cmd ( 0x06 );
    LCD_Send_Cmd ( 0x01 );
    LCD_Send_Cmd ( 0x02 );
	}

//******************************************************************************

void
DisplayStr ( char *sptr )
	{
    char x;
    
	for ( x = 0; x < 16; x++ )
		LCD_Send_Char ( *sptr++ );
	}

//******************************************************************************

void 
delay_seconds ( int delaytime)
	{
	for (;delaytime != 0; delaytime--)
		delay_ms(1000);
	}

//******************************************************************************

//char const msg [ 20 ] = {"G4FON SWR Bridge"};
char const msg [ 17 ]= "G4FON SWR Bridge";

main() 
	{
    char Forward;
    char Reverse;
    char AbsForward;
    char AbsReverse;
    char x;
	char op_string [ 18 ];
    char Range;
    char Mode;
    
    LCD_Init ();

	// power up delay

    delay_us ( 100 );

    //  output the logo...
    
	strcpy ( op_string, msg );
	DisplayStr ( op_string );

    //  show off the logo
    
	delay_seconds ( 1 );

    setup_port_a( RA0_RA1_ANALOG );
    setup_adc( ADC_CLOCK_INTERNAL );
    output_float ( INPUT_SW );
    
    //  main loop scanning the adc's and updating the display

    AbsForward = 0;
    Mode = 0;
    
    for (;;) 
        {
        //  realtime display of the reverse so that it can be tuned
        //  as quickly as possible.
        
        set_adc_channel ( 0 );
   		AbsReverse = Read_ADC ();

        //  read the forward and reflected voltages...

        set_adc_channel ( 1 );
   		x = Read_ADC ();

        //  fast attack, slow delay on the forward voltage...
        
        if ( x >= AbsForward )
            {
            //  update the value...
        
            AbsForward = x;

            //  multiply the results up so that we get a few bars to display...

            if (( AbsForward < 30 ) && ( AbsReverse < 30 ))
                {
                Forward = ABsForward * 8;
                Reverse = AbsReverse * 8; 
                }
            else if (( AbsForward < 60 ) && ( AbsReverse < 60 ))
                {
                Forward = ABsForward * 4;
                Reverse = AbsReverse * 4; 
                }
            else if (( AbsForward < 80 ) && ( AbsReverse < 80 ))
                {
                Forward = ABsForward * 3;
                Reverse = AbsReverse * 3; 
                }
            else if (( AbsForward < 128 ) && ( AbsReverse < 128 ))
                {
                Forward = ABsForward * 2;
                Reverse = AbsReverse * 2; 
                }

            Forward /= 16;

            if ( Forward > 12 )
                Forward = 12;
            }
        else
            {
            if ( Forward > 0 )
                Forward--;

            if ( Forward == 0 )
                AbsForward = 0;
            else
                AbsForward--;
            }

        //  home the cursor...

        LCD_Send_Cmd ( 0x02 );
        
        for ( x = 0; x < 16; x++ )
            op_string [ x ] = ' ';

        if ( AbsForward >= 100 )
            {
            op_string [ 13 ] = ( AbsForward / 100 )  + '0';
            op_string [ 14 ] = (( AbsForward / 10 ) % 10 )  + '0';
            op_string [ 15 ] = ( AbsForward % 10 )  + '0';
            }
        else if ( AbsForward > 0 )
            {
            op_string [ 14 ] = (( AbsForward / 10 ) % 10 )  + '0';
            op_string [ 15 ] = ( AbsForward % 10 )  + '0';
            }
        
            
        for ( x = 1; x < Forward; x++ )
            op_string [ x ] = 0xff;

        op_string [ 0 ] = 'F';

    	DisplayStr ( op_string );

        //  now do the reverse...
        
        for ( x = 0; x < 16; x++ )
            op_string [ x ] = ' ';

        if ( AbsReverse >= 100 )
            {
            op_string [ 13 ] = ( AbsReverse / 100 )  + '0';
            op_string [ 14 ] = (( AbsReverse / 10 ) % 10 )  + '0';
            op_string [ 15 ] = ( AbsReverse % 10 )  + '0';
            }
        else if ( AbsReverse > 0 )
            {
            op_string [ 14 ] = (( AbsReverse / 10 ) % 10 )  + '0';
            op_string [ 15 ] = ( AbsReverse % 10 )  + '0';
            }
            
        Reverse /= 16;

        if ( Reverse > 12 )
            Reverse = 12;
            
        for ( x = 0; x < Reverse; x++ )
            op_string [ x ] = 0xff;
            
        op_string [ 0 ] = 'R';
        
    	LCD_Send_Cmd ( LCD_2nd_Line );
    	DisplayStr ( op_string );
        }
	}

